<?php

namespace Drupal\edlp_migrate\Plugin\migrate\source;

use Drupal\migrate\Row;
use Drupal\migrate_drupal\Plugin\migrate\source\DrupalSqlBase;

/**
 * D6 Edlp Taxonomy Entree term source from database.
 *
 * @MigrateSource(
 *   id = "d6_edlp_taxonomy_term_entree"
 * )
 */
class D6EdlpTaxonomyTermEntree extends DrupalSqlBase {

  /**
   * {@inheritdoc}
   */
  public function query() {
    // drush_print('D6EdlpTaxonomyTermEntree query');

    $query = $this->select('term_data', 'td')
      ->fields('td')
      ->distinct()
      ->orderBy('td.tid');

    if (isset($this->configuration['bundle'])) {
      $query->condition('td.vid', (array) $this->configuration['bundle'], 'IN');
    }

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    $fields = [
      'tid' => $this->t('The term ID.'),
      'vid' => $this->t('Existing term VID'),
      'name' => $this->t('The name of the term.'),
      'description' => $this->t('The term description.'),
      'weight' => $this->t('Weight'),
      'parent' => $this->t("The Drupal term IDs of the term's parents."),
      'node_nid' => $this->t("The node's nid of entree's description."),
    ];
    if (isset($this->configuration['translations'])) {
      $fields['language'] = $this->t('The term language.');
      $fields['trid'] = $this->t('Translation ID.');
    }
    // print_r($fields);
    // print_r("\n");

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    // drush_print_r($row);
    // Find parents for this row.
    $parents = $this->select('term_hierarchy', 'th')
      ->fields('th', ['parent', 'tid'])
      ->condition('tid', $row->getSourceProperty('tid'))
      ->execute()
      ->fetchCol();
    $row->setSourceProperty('parent', $parents);

    // find node attached to this term for description
    $query = $this->select('content_type_entree', 'cte');
    // $query->fields('cte', ['nid', 'vid', 'field_entree_value']);
    $query->condition('cte.field_entree_value', $row->getSourceProperty('tid'));
    $query->join('node_revisions', 'nr', 'cte.nid = nr.nid');
    $query->addField('nr', 'body');
    $description = $query->execute()
      ->fetchField();

    // drush_print_r($node);
    $row->setSourceProperty('description', $description);

    return parent::prepareRow($row);
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    $ids['tid']['type'] = 'integer';
    return $ids;
  }

}
